import os
from timer import Timer
from hash import Hash
import tempfile
from matplotlib.pyplot import gcf, savefig


class Thumbnail(object):
    SIZE_PIXELS = 256

    @staticmethod
    def save_matplotlib_thumbnail(filename):
        fig = gcf()
        fig_size = fig.get_size_inches()
        Thumbnail.__resize_save_and_restore_matplotlib_thumbnail(filename,
                                                                 fig, fig_size)

    @staticmethod
    def __resize_save_and_restore_matplotlib_thumbnail(filename,
                                                       fig, fig_size):
        try:
            scale = Thumbnail.SIZE_PIXELS / (max(fig_size) * fig.dpi)
            fig.set_size_inches(scale * fig_size[0],
                                scale * fig_size[1])
            savefig(filename)
        finally:
            fig.set_size_inches(fig_size[0], fig_size[1])


class Figures(object):
    FIGURE_FORMAT = 'png'

    @staticmethod
    def add_output_figure(thumbnail_dir):
        with Timer() as timer:
            fighash = Figures.__current_figure_hash()
            Figures.__store_figure_thumbnail(thumbnail_dir, fighash)
        return {'hash': fighash, 'timing': timer.duration_ms()}

    @staticmethod
    def __current_figure_raw_png_data():
        try:
            tmp, temp_filename = tempfile.mkstemp(suffix='.png')
            os.close(tmp)  # os may prohibit saving to an already open file
            savefig(temp_filename)
            with open(temp_filename) as f:
                file_data = f.read()
        finally:
            os.remove(temp_filename)
        return file_data

    @staticmethod
    def __current_figure_hash():
        return Hash.hash(Figures.__current_figure_raw_png_data())

    @staticmethod
    def __save_current_figure_thumbnail(thumbnail_dir, filename):
        Thumbnail.save_matplotlib_thumbnail('%s/%s.%s' % (thumbnail_dir,
                                            filename, Figures.FIGURE_FORMAT))

    @staticmethod
    def __store_figure_thumbnail(thumbnail_dir, filename):
        if not os.path.exists(thumbnail_dir):
            os.makedirs(thumbnail_dir)
        Figures.__save_current_figure_thumbnail(thumbnail_dir, filename)



class Config(object):
    def load(self, conf_file_path):
        with open(conf_file_path) as f:
            content = f.readlines()
        self.__api_key = content[0].strip()

    def api_key(self):
        return self.__api_key



from timer import Timer
from hash import Hash


class Files(object):

    @staticmethod
    def generate_data_for_log(files_dict):
        data_for_log = {}
        for name, path in files_dict.items():
            data, read_duration = Files.__get_file_data_and_duration_of_read(
                path)
            file_hash, hash_duration = Files.__calc_hash_and_duration(data)
            size = len(data)
            data_for_log[name] = Files.__format_data_for_log(
                path,
                file_hash,
                size,
                read_duration,
                hash_duration)
        return data_for_log

    @staticmethod
    def __calc_hash_and_duration(file_data):
        with Timer() as timer:
            res = Hash.hash(file_data)
        return res, timer.duration_ms()

    @staticmethod
    def __get_file_data_and_duration_of_read(file_path):
        with Timer() as timer:
            with open(file_path) as f:
                file_data = f.read()
        return file_data, timer.duration_ms()

    @staticmethod
    def __format_data_for_log(path, file_hash, size, read_duration, hash_duration):
        return {'path': path,
                'hash': file_hash,
                'size': size,
                'timing': {
                    'file_read_ms': read_duration,
                    'hash_calc_ms': hash_duration},
                }
